﻿// Bravura : A music composition and audio synthesis framework.
//
// Copyright © 2011 Paul Vanukoff (pvanukoff@gmail.com)
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see http://www.gnu.org/licenses/.

using Bravura.Core.Common;
using Bravura.Core.MusicLib;
using Bravura.Core.PluginLib.Commands;

namespace Bravura.Contracts.Directives
{
	/// <summary>
	/// Sets the value of a run-time control for a plugin at a certain time.
	/// </summary>
	public class AutoChangeInputGain : AutoBase<ChangeInputGain>
	{
		#region Properties

		public double GainStart
		{
			get;
			set;
		}

		public double GainEnd
		{
			get;
			set;
		}

		#endregion

		public AutoChangeInputGain()
		{
			this.GainStart = 0.0;
			this.GainEnd = 0.0;
		}

		protected override ChangeInputGain Generate(ChangeInputGain dir, int relTick)
		{
			double k = 1.0 * (relTick + dir.Tick) / Delta;

			ChangeInputGain gen = new ChangeInputGain();
			gen.Tick = TickStart + dir.Tick + relTick;
			gen.PluginNum = dir.PluginNum;
			gen.ChannelNum = dir.ChannelNum;
			gen.Gain = GainStart + k * (GainEnd - GainStart);

			return gen;
		}
	}
}
