﻿// Bravura : A music composition and audio synthesis framework.
//
// Copyright © 2011 Paul Vanukoff (pvanukoff@gmail.com)
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see http://www.gnu.org/licenses/.

using Bravura.Core.Common;
using Bravura.Core.MusicLib;
using Bravura.Core.PluginLib.Commands;

namespace Bravura.Contracts.Directives
{
	/// <summary>
	/// Sets the value of a run-time control for a plugin at a certain time.
	/// </summary>
	public class AutoPluginControl : AutoBase<PluginControl>
	{
		#region Properties

		public double ValueStart
		{
			get;
			set;
		}

		public double ValueEnd
		{
			get;
			set;
		}

		#endregion

		public AutoPluginControl()
		{
			this.ValueStart = 0.0;
			this.ValueEnd = 0.0;
		}

		protected override PluginControl Generate(PluginControl dir, int relTick)
		{
			double k = 1.0 * (relTick + dir.Tick) / Delta;

			PluginControl gen = new PluginControl();
			gen.Tick = TickStart + dir.Tick + relTick;
			gen.Key = dir.Key;
			gen.Value = ValueStart + k * (ValueEnd - ValueStart);

			return gen;
		}
	}
}
