﻿// Bravura : A music composition and audio synthesis framework.
//
// Copyright © 2011 Paul Vanukoff (pvanukoff@gmail.com)
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see http://www.gnu.org/licenses/.

using Bravura.Core.Common;
using Bravura.Core.MusicLib;
using Bravura.Core.MusicLib.Commands;

namespace Bravura.Contracts.Directives
{
	/// <summary>
	/// Changes the tempo of a MusicEngine.
	/// </summary>
	public class AutoTempo : AutoBase<Tempo>
	{
		#region Properties

		public double TempoStart
		{
			get;
			set;
		}

		public double TempoEnd
		{
			get;
			set;
		}

		#endregion

		public AutoTempo()
		{
			this.TempoStart = 120.0;
			this.TempoEnd = 120.0;
		}

		protected override Tempo Generate(Tempo dir, int relTick)
		{
			double k = 1.0 * (relTick + dir.Tick) / Delta;

			Tempo gen = new Tempo();
			gen.Tick = TickStart + dir.Tick + relTick;
			gen.TempoValue = TempoStart + k * (TempoEnd - TempoStart);

			return gen;
		}
	}
}
