﻿// Bravura : A music composition and audio synthesis framework.
//
// Copyright © 2011 Paul Vanukoff (pvanukoff@gmail.com)
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see http://www.gnu.org/licenses/.

using Bravura.Core.Common;
using Bravura.Core.MusicLib;
using Bravura.Core.SynthLib.Commands;

namespace Bravura.Contracts.Directives
{
	/// <summary>
	/// Sets the gain of a input channel.
	/// </summary>
	public class ChangeInputGain : IDirective
	{
		private bool _inited = false;
		private SetInputGain _setInputGain = null;

		#region Properties

		public int Tick
		{
			get;
			set;
		}

		public int PluginNum
		{
			get;
			set;
		}

		public int ChannelNum
		{
			get;
			set;
		}

		public double Gain
		{
			get;
			set;
		}

		#endregion

		#region IDirective

		void IDirective.Execute(ICommandTarget target, int tick)
		{
			if (!_inited)
			{
				_setInputGain = new SetInputGain { PluginNum = this.PluginNum, ChannelNum = this.ChannelNum, Gain = this.Gain };
				_inited = true;
			}

			if (tick == Tick)
			{
				(_setInputGain as ICommand).Execute(target);
			}
		}

		#endregion
	}
}
