﻿// Bravura : A music composition and audio synthesis framework.
//
// Copyright © 2011 Paul Vanukoff (pvanukoff@gmail.com)
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see http://www.gnu.org/licenses/.

using Bravura.Core.Common;
using Bravura.Core.MusicLib;
using Bravura.Core.PluginLib.Commands;

namespace Bravura.Contracts.Directives
{
	/// <summary>
	/// Sets the value of a run-time setting for a plugin at a certain time.
	/// </summary>
	public class PluginSetting : IDirective
	{
		private bool _inited = false;
		private SetPluginSetting _setPluginSetting = null;

		#region Properties

		public int Tick
		{
			get;
			set;
		}

		public string Key
		{
			get;
			set;
		}

		public string Value
		{
			get;
			set;
		}

		#endregion

		#region IDirective

		void IDirective.Execute(ICommandTarget target, int tick)
		{
			if (!_inited)
			{
				_setPluginSetting = new SetPluginSetting { Key = this.Key, Value = this.Value };
				_inited = true;
			}

			if (tick == Tick)
			{
				(this as ICommand).Execute(target);
			}
		}

		#endregion
	}
}
