﻿// Bravura : A music composition and audio synthesis framework.
//
// Copyright © 2011 Paul Vanukoff (pvanukoff@gmail.com)
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see http://www.gnu.org/licenses/.

using System.Collections.Generic;
using System.Windows.Markup;
using Bravura.Core.Common;
using Bravura.Core.MusicLib;

namespace Bravura.Contracts.Directives
{
	/// <summary>
	/// Base class for a directive which repeats and optionally modifies a series of directives (a template).
	/// </summary>
	[ContentPropertyAttribute("Template")]
	public abstract class AutoBase<T> : IDirective where T : IDirective
	{
		private bool _inited = false;
		private List<T> _genDirs = new List<T>();
		private List<T> _template = new List<T>();

		#region Properties

		public int TickStart
		{
			get;
			set;
		}

		public int TickEnd
		{
			get;
			set;
		}

		public int Every
		{
			get;
			set;
		}

		public int Delta
		{
			get { return TickEnd - TickStart; }
		}

		public List<T> Template
		{
			get { return _template; }
		}

		#endregion

		protected AutoBase()
		{
			this.Every = 1;
		}

		protected abstract T Generate(T dir, int relTick);

		#region IDirective

		void IDirective.Execute(ICommandTarget target, int tick)
		{
			if (!_inited)
			{
				int n = (TickEnd - TickStart) / Every;
				for (int i = 0; i < n; i++)
				{
					foreach (T dir in _template)
					{
						_genDirs.Add(Generate(dir, i * Every));
					}
				}
				_inited = true;
			}

			if (TickStart <= tick && tick <= TickEnd)
			{
				foreach (T dir in _genDirs)
				{
					dir.Execute(target, tick);
				}
			}

			return;
		}

		#endregion 
	}
}
