﻿// Bravura : A music composition and audio synthesis framework.
//
// Copyright © 2011 Paul Vanukoff (pvanukoff@gmail.com)
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see http://www.gnu.org/licenses/.

using Bravura.Core.PluginLib;
using Bravura.Core.Common;

namespace Bravura.Contracts.PluginTypes
{
	/// <summary>
	/// An instrument generates audio in response to note-on, note-off and other commands.
	/// </summary>
	public interface IInstrument : IAudioPlugin
	{
		/// <summary>
		/// Set a setting for a voice.
		/// </summary>
		void SetVoiceSetting(int voice, string key, string value);

		/// <summary>
		/// Set a control for a voice.
		/// </summary>
		void SetVoiceControl(int voice, string key, double value);

		/// <summary>
		/// Send a note-on event to a voice.
		/// </summary>
		void SendVoiceNoteOn(int voice, double? sound, double? pitch, double? velocity);

		/// <summary>
		/// Send a note-off event to a voice.
		/// </summary>
		void SendVoiceNoteOff(int voice);
	}
}
