﻿// Bravura : A music composition and audio synthesis framework.
//
// Copyright © 2011 Paul Vanukoff (pvanukoff@gmail.com)
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see http://www.gnu.org/licenses/.

using System.IO;
using Bravura.Core.Common;
using Bravura.Core.MusicLib.Model;

namespace Bravura.Core.MusicLib
{
	/// <summary>
	/// Interface for a music engine.
	/// </summary>
	public interface IMusicEngine : ICommandTarget
	{
		/// <summary>
		/// Load a project from a MusicProject file.
		/// </summary>
		void Load(FileInfo projectFile);

		/// <summary>
		/// Load a project from a MusicProject object.
		/// </summary>
		void Load(MusicProject project);

		/// <summary>
		/// Render the currently loaded project.
		/// </summary>
		void Render();

		/// <summary>
		/// Render the currently loaded project from/to the specified beats.
		/// </summary>
		void Render(int from, int to);

		/// <summary>
		/// Sets the tempo (bpm).
		/// </summary>
		void SetTempo(double bpm);
	}
}
