﻿// Bravura : A music composition and audio synthesis framework.
//
// Copyright © 2011 Paul Vanukoff (pvanukoff@gmail.com)
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see http://www.gnu.org/licenses/.

using System.Diagnostics;

namespace Bravura.Core.MusicLib
{
	/// <summary>
	/// Interface for a music engine host.
	/// </summary>
	public interface IMusicEngineHost
	{
		/// <summary>
		/// Query the host if we should continue the current operation.
		/// </summary>
		bool ShouldContinue();

		/// <summary>
		/// Inform the host of the progress of the current operation.
		/// </summary>
		void LogProgress(int progress, int total);

		/// <summary>
		/// Log a message from a music engine to the host.
		/// </summary>
		void LogMessage(IMusicEngine source, TraceLevel severity, string message);
	}
}
