﻿// Bravura : A music composition and audio synthesis framework.
//
// Copyright © 2011 Paul Vanukoff (pvanukoff@gmail.com)
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see http://www.gnu.org/licenses/.

using System;
using System.ComponentModel;
using System.Reflection;

namespace Bravura.Core.Common
{
	/// <summary>
	/// Extension methods.
	/// </summary>
	public static class Extensions
	{
		#region Methods

		/// <summary>
		/// Determines if a public property with the given name exists.
		/// </summary>
		/// <returns>Returns true on success.</returns>
		public static bool HasProperty(this object obj, string name)
		{
			Type pt = obj.GetType();
			PropertyInfo pi = pt.GetProperty(name);
			return (pi != null);
		}

		/// <summary>
		/// Sets a value of a public property on an object.
		/// </summary>
		/// <remarks>
		/// Uses reflection.
		/// </remarks>
		/// <returns>Returns true on success.</returns>
		public static bool SetPropertyValue(this object obj, string name, object value)
		{
			Type pt = obj.GetType();
			PropertyInfo pi = pt.GetProperty(name);
			if (pi != null)
			{
				if (value.GetType() == pi.PropertyType)
				{
					pi.SetValue(obj, value, null);
					return true;
				}
				else
				{
					TypeConverter tc = TypeDescriptor.GetConverter(pi.PropertyType);
					if (tc.CanConvertFrom(value.GetType()) && tc.CanConvertTo(pi.PropertyType))
					{
						object newVal = tc.ConvertFrom(value);
						pi.SetValue(obj, newVal, null);
						return true;
					}
				}
			}
			return false;
		}

		#endregion
	}
}
