﻿// Bravura : A music composition and audio synthesis framework.
//
// Copyright © 2011 Paul Vanukoff (pvanukoff@gmail.com)
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see http://www.gnu.org/licenses/.

using System.Collections.Generic;
using System.Windows.Markup;
using Bravura.Core.Common;

namespace Bravura.Core.SynthLib.Model
{
	/// <summary>
	/// Data that the SynthEngine will configure a plugin with after instantiation.
	/// </summary>
	[ContentPropertyAttribute("Values")]
	public class ConfigItem : IConfigItem
	{
		private List<IConfigItem> _values = new List<IConfigItem>();

		/// <summary>
		/// Name of the configuration item.
		/// </summary>
		public string Name
		{
			get;
			set;
		}

		/// <summary>
		/// Value as a string.
		/// </summary>
		public string Value
		{
			get;
			set;
		}

		/// <summary>
		/// Used for more complex configuration.
		/// </summary>
		public List<IConfigItem> Values
		{
			get { return _values; }
		}

		#region IConfigItem

		string IConfigItem.GetName()
		{
			return this.Name;
		}

		string IConfigItem.GetValue()
		{
			return this.Value;
		}

		List<IConfigItem> IConfigItem.GetChildItems()
		{
			return this.Values;
		}

		#endregion
	}
}
