﻿// Bravura : A music composition and audio synthesis framework.
//
// Copyright © 2011 Paul Vanukoff (pvanukoff@gmail.com)
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see http://www.gnu.org/licenses/.

using System.Collections.Generic;
using Bravura.Core.Common;

namespace Bravura.Core.SynthLib.Model
{
	/// <summary>
	/// Represents plugin instantiation and processing information.
	/// </summary>
	public class PluginInfo
	{
		private List<IConfigItem> _configuration = new List<IConfigItem>();

		/// <summary>
		/// Instance number.
		/// </summary>
		public int Num
		{
			get;
			set;
		}

		/// <summary>
		/// Instance name.
		/// </summary>
		public string Name
		{
			get;
			set;
		}

		/// <summary>
		/// Qualified name of the plugin type.
		/// </summary>
		public string TypeName
		{
			get;
			set;
		}

		/// <summary>
		/// Plugin configuration.
		/// </summary>
		public List<IConfigItem> Configuration
		{
			get { return _configuration; }
		}

		/// <summary>
		/// Major processing order.
		/// </summary>
		public short MajorOrder
		{
			get;
			set;
		}

		/// <summary>
		/// Minor processing order.
		/// </summary>
		public short MinorOrder
		{
			get;
			set;
		}

		/// <summary>
		/// Calculated processing order.
		/// </summary>
		public int ProcessOrder
		{
			get { return (MajorOrder << 16) + MinorOrder; }
		}
	}
}
